<?php
/*
Plugin Name: GoogleMaps Shortcode
Plugin URI: http://tenderfeel.xsrv.jp/wordpress/799/
Description: GoogleMapsを表示するショートコード機能を追加します。
Author: Tenderfeel
Version: 1.0
Author URI: http://tenderfeel.xsrv.jp/
*/

/*****************************************

  Google Maps ShortCode
  by:tenderfeel(tenderfeel@gmail.com)

-------------------------------------------
  使用方法
-------------------------------------------
  ★基本★
  
  記事本文内で[Gmap] と書く。
  
  属性の設定は次のように。
  [Gmap width="500px" lat="35.689184"]

  間に何か文字とかを入れた場合はタブ内に表示。
　
  [Gmap]

  <em>Hellow</em>

  何かコメントを入れてください

  [/Gmap]
  
  ★属性の指定★
  
	[Gmap width="100%" height="600px" overview="false" lat="35.689184" lng="139.691648"]
	
	<em>東京都庁</em>
	〒163-8001 東京都新宿区西新宿２丁目８−１
	<ul>
		<li> TEL:03-5321-1111</li>
		<li>FAX:03-5321-1111</li>
	</ul>
	
	[/Gmap]
  
  ★マーカーを増やす★
  
	[Gmap width="100%" height="600px" overview="false"]
	
		[marker name="都庁" lat="35.689184" lng="139.691648"]
		[tab title="所在地"]
		<em>東京都庁</em>
		〒163-8001 東京都新宿区西新宿２丁目８−１
		<ul>
			<li> TEL:03-5321-1111</li>
			<li>FAX:03-5321-1111</li>
		</ul>
		[/tab]
		
		[tab title="外観"]
		<img class="alignnone size-thumbnail wp-image-83" title="Sea" src="http://localhost/wordpress/wp-content/uploads/2009/04/reef_10-100x100.jpg" alt="" width="100" height="100" />
		
		[/tab]
		[/marker]
		
		[marker name="東京タワー" lat="35.65861" lng="139.745447"]
		東京タワー
		[/marker]
		[marker name="東京大学" lat="35.713428" lng="139.762308"]
		[maxim title="東大情報"]
		<img class="alignnone size-medium wp-image-95" title="7e575e76" src="http://localhost/wordpress/wp-content/uploads/2009/06/7e575e76-233x300.jpg" alt="" width="233" height="300" />
		[/maxim]
		[/marker]
		
	[/Gmap]
	
-------------------------------------------
  属性一覧
-------------------------------------------
  カッコはデフォルト値。値はすべて文字列で。
  デフォルトの座標は東京都庁になっています。
  変更する場合は$defaultの値を修正する。

  @width ---------- (100%) 地図の横幅
  @height --------- (500px) 地図の縦幅
  @lat ------------ (35.689184) マーカーlat座標
  @lng ------------ (139.691648) マーカーlng座標
  @zoom ----------- (10) 拡大率[0～20]
  @marker --------- (true)　マーカーを表示するかどうか
  @zoomControler -- (false) ズームコントローラーを表示するかどうか
  @controler ------ (true)　コントローラーを表示するかどうか
  @navLabel ------- (false)　ナビゲーションレベルコントローラーを表示するかどうか
  @maptype -------- (3)　地図タイプ切り替え[1 or 2 or 3]
  @scale ---------- (false) スケールコントローラを表示するかどうか
  @overview ------- (true)　右下の縮小地図
  @wheelZoom ------ (false)　ホイールでのズーム操作
  @clickZoom ------ (true)　ダブルクリックでのズーム操作
  @html ----------- ("")　情報ウィンドウのHTML
  
-------------------------------------------
  クラスのプロパティ
-------------------------------------------

  $atts --- 属性のデフォルト値
  $google_apikey --- APIKEY
  
*******************************************/


class Googlemap_Shortcode {
	var $atts = array(
		'width'=>'100%',
		'height'=>'500px',
		'lat' => '35.689184',
		'lng' => '139.691648',
		'zoom'=> '10',
		'marker'=>'true',
		'zoomcontroler'=>'false',
		'controler'=>'true',
		'navlabel'=>'false',
		'maptype'=>'3',
		'scale'=>'false',
		'overview'=>'true',
		'wheelzoom'=>'false',
		'clickzoom'=>'true',
		'id'=>'',
		'html'=>''
		);
	
	var $google_apikey = "ADD_YOUR_API_KEY";
	
	var $nav = array();
	
	var $tab = array();
	
	var $maxim = null;
	
	function setAPIKey() {
		$this->google_apikey = get_option('googlemaps_api_key')||$this->google_apikey;
		if (!$this->google_apikey) $this->google_apikey = get_option('yf_google_api_key');
		if (!$this->google_apikey) $this->google_apikey= (defined('GM_ANYWHERE_API_KEY') ? GM_ANYWHERE_API_KEY : false);
		return $this->google_apikey;
	}
	
	function __construct(){
		
		$this->setAPIKey();
	}
	
	function Googlemap_Shortcode(){
		$this->__construct();
	}
	
	function google_loader(){
			$script = '<script type="text/javascript" src="http://www.google.com/jsapi?key='.$this->google_apikey.'"></script>'."\n";
			$script.='<script type="text/javascript" src="'.get_bloginfo('template_directory').'/gmap.js"></script>'."\n";
			print $script;
	}
	
	//var gmConfig
	function _js_config(){
		
		$script = "var gmConfig = {'gmLat':".$this->atts['lat'].",'gmLng':".$this->atts['lng'].",'gmDivID':'".$this->atts['id']."','gmScope':".$this->atts['zoom'].",'markerAdd':".$this->atts['marker'].",";
		$script .="'LMap_Controler':".$this->atts['controler'].",";
		$script .="'SmallZoomControl':".$this->atts['zoomcontroler'].",'NavLabelControl':".$this->atts['navlabel'].",";
		$script .="'MapType_Controler':".$this->atts['maptype'].",";
		$script .="'Scale_Controler':".$this->atts['scale'].",'OV_Controler':".$this->atts['overview'].",'WheelZoom':".$this->atts['wheelzoom'].",'DClickZoom':".$this->atts['clickzoom'];
			if($this->atts['html']!="")
				$script .= ",'html':'".$this->atts['html']."'";
		$script .= "};\n";
	
		return $script;
	}
	
	//変換
	function convert_shortcode($atts, $content){
		global $post;
		
		add_action('wp_footer', array($this, 'google_loader'));
		
		
		$this->atts["id"] = uniqid('gmap-');
		
		$this->atts = shortcode_atts($this->atts, $atts);
		
		$source .= "<script type=\"text/javascript\">\n//<![CDATA[\n";
																  
		if($content!="" && strpos($content,"[/marker]")===false){
			
			$this->atts['html'] = $this->_replace_trash($content);
			
		}
		
		$source .= $this->_js_config();//print gmConfig
		
		preg_match_all("/\[marker(.*?)\](.*?)\[\/marker\]?/sm",$content,$match);
		
		if($content!=null){
			$reg = array("/^(.*?)\{?/i", "/}?,$/i", "|<p>(\s*)<\/p>|","/,},/");
			$rex = array("{", "}","","},");
			$ccc = "";
			for($i = 0; $i < count($match[1]); $i++){
				$ccc .= balanceTags($this->_marker_output($i,shortcode_parse_atts($match[1][$i]),$match[2][$i]),true);
			}
			if($ccc!="") $ccc = preg_replace($reg,$rex,$ccc);
				
			$source .= "var area =[".$ccc."];\n";
		}else{
			$source .= "var area =[];\n";
		}
		
		
		$source .="//]] >\n</script>\n";
		$source .='<div id="'.$this->atts['id'].'" style="width:'.$this->atts['width'].';height:'.$this->atts['height'].'"></div>';
		
		$source .= $this->_nav_output();
		
		return $source; 
	}
	
	function _marker_output($z,$atts, $content){
		$atts = shortcode_atts(array(
		'name'=>'東京都庁',
		'lat' => '35.689184',
		'lng' => '139.691648',
		'blowup'=>"false",
		'html'=>''
		), $atts);
		
		$this->nav[] = $atts["name"];
		
		preg_match("/(.*?)\[maxim(.*?)\](.*)?\[\/maxim\]?/s",$content,$maxims);
		
		preg_match_all("/\[tab(.*?)\](.*?)\[\/tab\]?/s",$content,$tabs);
		
		$out .=  "{'name':'".$atts['name']."','lat':".$atts['lat'].",'lng':".$atts['lng'].",'blowup':".$atts['blowup'].",";
		
		if(count($maxims[0])==0 && count($tabs[0])==0){
			$out .= "'html':'".$this->_replace_trash($content)."'";
		}
		
		if(count($maxims[0])!=0){
			$maxims[1] = $this->_replace_trash($maxims[1]);
			$atts["html"] = ($maxims[1]=="")? "右端のズームボタン[+]をクリックしてください" : $maxims[1];
			$maxim .= $this->_maxim(shortcode_parse_atts($maxims[2]),$maxims[3]);
			
			$out .= "'html':'".$atts["html"]."',";
			$out .= $maxim;
		}
		
		if(count($tabs[0])!=0){
			for($i = 0; $i < count($tabs[1]); $i++){
				$tab[]= $this->_tab(shortcode_parse_atts($tabs[1][$i]),$tabs[2][$i]);
			}

			if($tab!= NULL && count($maxims)==0){
				$out .= "'tabs':[";
					foreach($tab as $i => $val){
						$out .= "{'title':'".$val["title"]."','html':'".$val["html"]."'}";
						
						if($i < count($tabs[0])-1) $out.=",";
					}
				$out .= "]";
			}
		}
		
		$out .= "},";
		 
		return $out;
	}
	//拡大
	function _maxim($atts, $content){
		$atts = shortcode_atts(array(
		'title'=>'',
		'html'=>''
		), $atts);
		
		if($content!=null){
			$atts['html'] = $this->_replace_trash($content);
		}
	
		return "'maxtitle':'".$atts["title"]."', 'maxhtml':'".$atts["html"]."'";
	}
	//タブ
	function _tab($atts, $content){
		$atts = shortcode_atts(array(
		'title'=>'',
		'html'=>''
		), $atts);
		
		if($content!=null){
			$atts['html'] = $this->_replace_trash($content);
		}
		
		return array("title"=>$atts['title'], "html"=>$atts['html']);
	}
	//ナビ
	function _nav_output(){
		
		$nav = '<ul class="gmap-nav '.$this->atts['id'].'">';
		foreach($this->nav as $i => $name) {
			$nav.= '<li><a href="#" onclick="panTo('.$i.');return false;">'.$name.'</a></li>';
		}
		$nav .= '</ul>';
		
		return $nav;
	}
	
	function _replace_trash($txt){
		$txt = do_shortcode($txt);
		$txt = preg_replace(array("/\r|\n/","/^[\s\r\n]*<br \/>/im","/<br \/>[\s\r\n]*$/","/<p>\s*<\/p>/i"),"",balanceTags(trim($txt),true));
		
		return $txt;
	}
}

$google_maps_shortcode = new Googlemap_Shortcode();

add_shortcode('Gmap', array($google_maps_shortcode , 'convert_shortcode'));
?>
